#!/usr/bin/env python
"""
Unit tests for the output of see for various types of object.

"""
import itertools

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import see


def union(*sets):
    return set(itertools.chain(*sets))


SIGN_OPS = set(['+obj', '-obj'])

NUMBER_OPS = set('+ - * / // % **'.split())
NUMBER_ASSIGN_OPS = set()

BITWISE_OPS = set('<< >> & ^ | ~'.split())
BITWISE_ASSIGN_OPS = set(op + '=' for op in BITWISE_OPS)

COMPARE_OPS = set('< <= == != > >='.split())

MATRIX_OPS = set(['@'])
MATRIX_ASSIGN_OPS = set(['@='])

ALL_OPS = union(
    SIGN_OPS,
    NUMBER_OPS,
    NUMBER_ASSIGN_OPS,
    BITWISE_OPS,
    BITWISE_ASSIGN_OPS,
    COMPARE_OPS,
    MATRIX_OPS,
    MATRIX_ASSIGN_OPS,
)


class TestSeeOutput(unittest.TestCase):

    def check_ops(self, obj_type, expected_ops, see_output):
        for op in ALL_OPS:
            if op in expected_ops:
                self.assertIn(
                    op, see_output,
                    'expected %s to support %s' % (obj_type, op))
            else:
                self.assertNotIn(
                    op, see_output,
                    'expected %s not to support %s' % (obj_type, op))

    def test_int(self):
        obj = 1
        lit_ops = union(
            SIGN_OPS,
            NUMBER_OPS,
            BITWISE_OPS,
            COMPARE_OPS,
        )
        obj_ops = union(
            lit_ops,
        )

        lit_see = see.see(1)
        obj_see = see.see(obj)

        self.check_ops('int literal', lit_ops, lit_see)
        self.check_ops('int object', obj_ops, obj_see)

    def test_float(self):
        obj = 1.0
        lit_ops = union(
            SIGN_OPS,
            NUMBER_OPS,
            COMPARE_OPS,
        )
        obj_ops = union(
            lit_ops,
        )

        lit_see = see.see(1.0)
        obj_see = see.see(obj)

        self.check_ops('float literal', lit_ops, lit_see)
        self.check_ops('float object', obj_ops, obj_see)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Unit tests for getting information about the terminal.

There are separate test cases to simulate unsupported environments (not
Unixlike or Windows), where information about the terminal is not easily
available. To do this, we prevent Python from importing some modules while it
loads see.

"""
import platform
import sys
from imp import reload

try:
    import unittest2 as unittest
except ImportError:
    import unittest

try:
    import unittest.mock as mock
except ImportError:
    import mock

try:
    import builtins  # Python 3
except ImportError:
    import __builtin__ as builtins  # Python 2

import see


MOCK_EXCLUDE_MODULES = (
    'ctypes',
    'fcntl',
    'termios',
)

PY3 = sys.version_info >= (3, 0)

REAL_IMPORT = builtins.__import__


def mock_import(name,
                globals=None if PY3 else {},
                locals=None if PY3 else {},
                fromlist=None if PY3 else [],
                level=0 if PY3 else -1):
    if name in MOCK_EXCLUDE_MODULES:
        raise ImportError
    return REAL_IMPORT(name, globals, locals, fromlist, level)


class TestSupportedTerminal(unittest.TestCase):

    def setUp(self):
        self.system = platform.system()
        self.windows = (self.system == 'Windows')

    def test_system(self):
        self.assertTrue(self.system, 'System/OS name could not be determined')

    def test_import_success(self):
        if self.windows:
            self.assertIsNone(see.fcntl)
            self.assertIsNone(see.termios)
            self.assertIsNotNone(see.windll)
            self.assertIsNotNone(see.create_string_buffer)
        else:
            self.assertIsNotNone(see.fcntl)
            self.assertIsNotNone(see.termios)
            self.assertIsNone(see.windll)
            self.assertIsNone(see.create_string_buffer)

    def test_term_width(self):
        width = see.term_width()

        self.assertIsNotNone(width)
        self.assertGreater(width, 0)

    def test_ioctl_fail(self):
        if self.windows:
            return

        with mock.patch('see.fcntl.ioctl', side_effect=IOError('')) as patch:
            width = see.term_width()

            self.assertIsNone(width)

    def test_indent_to_prompt(self):
        if hasattr(sys, 'ps1'):
            self.fail('expected sys.ps1 to be absent during unit testing')

        # Arrange
        sys.ps1 = '>>>>> '

        # Act
        out = see.see()
        indent = len(str(out)) - len(str(out).lstrip())

        # Assert
        self.assertEqual(indent, len(sys.ps1))


class TestMockWindowsTerminal(unittest.TestCase):

    def setUp(self):
        builtins.__import__ = mock_import
        reload(see)

    def tearDown(self):
        builtins.__import__ = REAL_IMPORT
        reload(see)


class TestMockUnsupportedTerminal(unittest.TestCase):

    def setUp(self):
        builtins.__import__ = mock_import
        reload(see)

    def tearDown(self):
        builtins.__import__ = REAL_IMPORT
        reload(see)

    def test_import_fail(self):
        self.assertIsNone(see.fcntl)
        self.assertIsNone(see.termios)

    def test_term_width_not_available(self):
        term_width = see.term_width()

        self.assertIsNone(term_width)

    def test_default_line_width(self):
        line_width = see.line_width()

        self.assertEqual(line_width, see.DEFAULT_LINE_WIDTH)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Unit tests for see.py

"""
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import see


class ObjectWithAttributeError(object):

    @property
    def bad_attribute(self):
        raise Exception


class ObjectWithLongAttrName(dict):

    def dajsgkljsdgklajsdgkljdsgkldsjaglkjasdgkldajsgkl(self):
        pass


class TestSee(unittest.TestCase):

    def test_line_width(self):
        # Arrange
        default_width = 1
        max_width = 1

        # Act
        width = see.line_width(default_width, max_width)
        width_no_args = see.line_width()

        # Assert
        self.assertIsInstance(width, int)
        self.assertEqual(width, 1)
        self.assertLessEqual(width_no_args, see.MAX_LINE_WIDTH)

    def test_regex_filter(self):
        # Arrange
        names = ["george", "helen"]
        pat_wildcard = "e.*g"
        pat_start = "^h"
        pat_end = "n$"

        # Act
        out_wildcard = see.regex_filter(names, pat_wildcard)
        out_start = see.regex_filter(names, pat_start)
        out_end = see.regex_filter(names, pat_end)

        # Assert
        self.assertIsInstance(out_wildcard, tuple)
        self.assertIsInstance(out_start, tuple)
        self.assertIsInstance(out_end, tuple)
        self.assertEqual(out_wildcard, ("george",))
        self.assertEqual(out_start, ("helen",))
        self.assertEqual(out_end, ("helen",))

    def test_fn_filter(self):
        # Arrange
        names = ["george", "helen"]
        pat_wildcard = "*or*"
        pat_single = "h?l?n"
        pat_partial = "e*"

        # Act
        out_wildcard = see.fn_filter(names, pat_wildcard)
        out_single = see.fn_filter(names, pat_single)
        out_partial = see.fn_filter(names, pat_partial)

        # Assert
        self.assertIsInstance(out_wildcard, tuple)
        self.assertIsInstance(out_single, tuple)
        self.assertIsInstance(out_partial, tuple)
        self.assertEqual(out_wildcard, ("george",))
        self.assertEqual(out_single, ("helen",))
        self.assertEqual(out_partial, ())

    def test_see_with_no_args(self):
        # Act
        out = see.see()
        default_arg = see._LOCALS

        # Assert
        self.assertIsInstance(out, see._SeeOutput)
        self.assertEqual(repr(default_arg), 'anything')

    def test_see_accessor_raises_exception(self):
        # Arrange
        normal_obj = 1
        err_obj = ObjectWithAttributeError()

        # Act
        normal_see = see.see(normal_obj)
        err_see = see.see(err_obj)

        # Assert
        self.assertTrue(not any(attr.endswith('?')
                                for attr in normal_see))
        self.assertTrue(any(attr.endswith('?')
                            for attr in err_see))
        self.assertIn('.bad_attribute?', err_see)

    def test_see_regex_filter(self):
        # Arrange
        obj = []

        # Act
        out = see.see(obj, r='[aeiou]{2}')

        # Assert
        self.assertIn('.count()', out)
        self.assertNotIn('.pop()', out)

    def test_see_pattern_filter(self):
        # Arrange
        obj = []

        # Act
        out = see.see(obj, '*()')

        # Assert
        self.assertIn('.count()', out)
        self.assertNotIn('+=', out)

    def test_see_repr(self):
        # Act
        out = see.see()

        # Assert
        self.assertEqual(str(out), out.__repr__())

    def test_see_justify_attributes(self):
        # Arrange
        obj = ObjectWithLongAttrName()

        # Act
        out = see.see(obj)
        col_width = see.column_width(out)
        padded = [see.justify_token(tok, col_width) for tok in out]
        lens = sorted(map(len, padded))
        factors = tuple(float(num) / lens[0] for num in lens[1:])

        # Assert
        self.assertNotEqual(lens[0], lens[-1],
                            'Expected differing column widths')
        self.assertTrue(any(factors))
        self.assertTrue(all(int(factor) == factor for factor in factors),
                        'Irregular column widths')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
see
dir for humans

    >>> from see import see
    >>> help(see)

Copyright (c) 2009-2016 Liam Cooke
http://araile.github.io/see/

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY LIAM COOKE "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

"""
import fnmatch
import inspect
import math
import platform
import re
import struct
import sys
import textwrap
import unicodedata

try:
    if platform.system() == 'Windows':
        from ctypes import windll, create_string_buffer
        fcntl, termios = None, None
    else:
        import fcntl
        import termios
        windll, create_string_buffer = None, None
except ImportError:
    fcntl, termios = None, None
    windll, create_string_buffer = None, None


__all__ = ['see']

__author__ = 'Liam Cooke'
__contributors__ = 'See AUTHORS.rst'
__version__ = '1.3.2'
__copyright__ = 'Copyright (c) 2009-2016 Liam Cooke'
__license__ = 'BSD License'


DEFAULT_LINE_WIDTH = 78
MAX_LINE_WIDTH = 120

PY_300 = sys.version_info >= (3, 0)
PY_301 = sys.version_info >= (3, 0, 1)
PY_350 = sys.version_info >= (3, 5, 0)


def term_width():
    """
    Return the column width of the terminal, or None if it can't be determined.

    """
    if fcntl and termios:
        try:
            winsize = fcntl.ioctl(0, termios.TIOCGWINSZ, '    ')
            _, width = struct.unpack('hh', winsize)
            return width
        except IOError:
            pass
    elif windll and create_string_buffer:
        stderr_handle, struct_size = -12, 22
        handle = windll.kernel32.GetStdHandle(stderr_handle)
        csbi = create_string_buffer(struct_size)
        res = windll.kernel32.GetConsoleScreenBufferInfo(handle, csbi)
        if res:
            (_, _, _, _, _, left, _, right, _,
             _, _) = struct.unpack('hhhhHhhhhhh', csbi.raw)
            return right - left + 1


def line_width(default_width=DEFAULT_LINE_WIDTH, max_width=MAX_LINE_WIDTH):
    """
    Return the ideal column width for see() output, taking the terminal width
    into account to avoid wrapping.

    """
    width = term_width()
    if width:
        return min(width, max_width)
    else:
        return default_width


def char_width(char):
    """
    Get the display length of a unicode character.

    """
    if ord(char) < 128:
        return 1
    elif unicodedata.east_asian_width(char) in ('F', 'W'):
        return 2
    elif unicodedata.category(char) in ('Mn',):
        return 0
    else:
        return 1


def display_len(text):
    """
    Get the display length of a string. This can differ from the character
    length if the string contains wide characters.

    """
    text = unicodedata.normalize('NFD', text)
    return sum(char_width(char) for char in text)


def regex_filter(names, pat):
    """
    Return a tuple of strings that match the regular expression pattern.

    """
    pat = re.compile(pat)

    def match(name, fn=pat.search):
        return fn(name) is not None

    return tuple(filter(match, names))


def fn_filter(names, pat):
    """
    Return a tuple of strings that match a shell-style pattern.

    """
    def match(name, fn=fnmatch.fnmatch, pat=pat):
        return fn(name, pat)

    return tuple(filter(match, names))


class SeeError(Exception):
    pass


def column_width(tokens):
    """
    Return a suitable column width to display one or more strings.

    """
    get_len = display_len if PY_300 else len
    lens = sorted(map(get_len, tokens or [])) or [0]
    width = lens[-1]

    # adjust for disproportionately long strings
    if width >= 18:
        most = lens[int(len(lens) * 0.9)]
        if most < width + 6:
            return most

    return width


def justify_token(tok, col_width):
    """
    Justify a string to fill one or more columns.

    """
    tok_len = display_len(tok) if PY_300 else len(tok)
    diff_len = tok_len - len(tok) if PY_300 else 0

    cols = (int(math.ceil(float(tok_len) / col_width))
            if col_width < tok_len + 4 else 1)

    if cols > 1:
        return tok.ljust((col_width * cols) + (4 * cols) - diff_len)
    else:
        return tok.ljust(col_width + 4 - diff_len)


class _SeeOutput(tuple):
    """
    Tuple-like object with a pretty string representation.

    """
    def __new__(self, actions=None):
        return tuple.__new__(self, actions or [])

    def __repr__(self):
        col_width = column_width(self)

        padded = [justify_token(tok, col_width) for tok in self]
        if hasattr(sys, 'ps1'):
            get_len = display_len if PY_300 else len
            indent = ' ' * get_len(sys.ps1)
        else:
            indent = '    '

        return textwrap.fill(''.join(padded), line_width(),
                             initial_indent=indent,
                             subsequent_indent=indent)


class _SeeDefault(object):

    def __repr__(self):
        return 'anything'

_LOCALS = _SeeDefault()


def see(obj=_LOCALS, pattern=None, r=None):
    """
    Inspect an object. Like the dir() builtin, but easier on the eyes.

    Keyword arguments (all optional):
    obj -- object to be inspected
    pattern -- shell-style search pattern (e.g. '*len*')
    r -- regular expression

    If obj is omitted, objects in the current scope are listed instead.

    Some unique symbols are used:

        .*      implements obj.anything
        []      implements obj[key]
        in      implements membership tests (e.g. x in obj)
        +obj    unary positive operator (e.g. +2)
        -obj    unary negative operator (e.g. -2)
        ?       raised an exception

    """
    use_locals = obj is _LOCALS
    actions = []
    dot = not use_locals and '.' or ''
    name = lambda a, f: ''.join((dot, a, suffix(f)))

    def suffix(f):
        if isinstance(f, SeeError):
            return '?'
        elif hasattr(f, '__call__'):
            return '()'
        else:
            return ''

    if use_locals:
        obj.__dict__ = inspect.currentframe().f_back.f_locals
    attrs = dir(obj)
    if not use_locals:
        for var, symbol in SYMBOLS:
            if var not in attrs or symbol in actions:
                continue
            elif var == '__doc__':
                if not obj.__doc__ or not obj.__doc__.strip():
                    continue
            actions.append(symbol)

    for attr in filter(lambda a: not a.startswith('_'), attrs):
        try:
            prop = getattr(obj, attr)
        except (AttributeError, Exception):
            prop = SeeError()
        actions.append(name(attr, prop))

    if pattern is not None:
        actions = fn_filter(actions, pattern)
    if r is not None:
        actions = regex_filter(actions, r)

    return _SeeOutput(actions)


SYMBOLS = tuple(filter(lambda x: x[0], (
    # callable
    ('__call__', '()'),

    # element/attribute access
    ('__getattr__', '.*'),
    ('__getitem__', '[]'),
    ('__setitem__', '[]'),
    ('__delitem__', '[]'),

    # iteration
    ('__enter__', 'with'),
    ('__exit__', 'with'),
    ('__contains__', 'in'),

    # operators
    ('__add__', '+'),
    ('__radd__', '+'),
    ('__iadd__', '+='),
    ('__sub__', '-'),
    ('__rsub__', '-'),
    ('__isub__', '-='),
    ('__mul__', '*'),
    ('__rmul__', '*'),
    ('__imul__', '*='),
    (PY_350 and '__matmul__', '@'),
    (PY_350 and '__rmatmul__', '@'),
    (PY_350 and '__imatmul__', '@='),
    (not PY_300 and '__div__', '/'),
    (not PY_301 and '__rdiv__', '/'),
    ('__truediv__', '/'),
    ('__rtruediv__', '/'),
    ('__floordiv__', '//'),
    ('__rfloordiv__', '//'),
    (not PY_300 and '__idiv__', '/='),
    ('__itruediv__', '/='),
    ('__ifloordiv__', '//='),
    ('__mod__', '%'),
    ('__rmod__', '%'),
    ('__divmod__', '%'),
    ('__imod__', '%='),
    ('__pow__', '**'),
    ('__rpow__', '**'),
    ('__ipow__', '**='),
    ('__lshift__', '<<'),
    ('__rlshift__', '<<'),
    ('__ilshift__', '<<='),
    ('__rshift__', '>>'),
    ('__rrshift__', '>>'),
    ('__irshift__', '>>='),
    ('__and__', '&'),
    ('__rand__', '&'),
    ('__iand__', '&='),
    ('__xor__', '^'),
    ('__rxor__', '^'),
    ('__ixor__', '^='),
    ('__or__', '|'),
    ('__ror__', '|'),
    ('__ior__', '|='),
    ('__pos__', '+obj'),
    ('__neg__', '-obj'),
    ('__invert__', '~'),
    ('__lt__', '<'),
    (not PY_301 and '__cmp__', '<'),
    ('__le__', '<='),
    (not PY_301 and '__cmp__', '<='),
    ('__eq__', '=='),
    (not PY_301 and '__cmp__', '=='),
    ('__ne__', '!='),
    (not PY_301 and '__cmp__', '!='),
    ('__gt__', '>'),
    (not PY_301 and '__cmp__', '>'),
    ('__ge__', '>='),
    (not PY_301 and '__cmp__', '>='),

    # built-in functions
    ('__abs__', 'abs()'),
    (PY_300 and '__bool__' or '__nonzero__', 'bool()'),
    ('__complex__', 'complex()'),
    (PY_300 and '__dir__', 'dir()'),
    ('__divmod__', 'divmod()'),
    ('__rdivmod__', 'divmod()'),
    ('__float__', 'float()'),
    ('__hash__', 'hash()'),
    ('__doc__', 'help()'),
    (PY_300 and '__index__' or '__hex__', 'hex()'),
    ('__int__', 'int()'),
    ('__iter__', 'iter()'),
    ('__len__', 'len()'),
    (not PY_300 and '__long__', 'long()'),
    (PY_300 and '__index__' or '__oct__', 'oct()'),
    ('__repr__', 'repr()'),
    ('__reversed__', 'reversed()'),
    (PY_300 and '__round__', 'round()'),
    ('__str__', 'str()'),
    (PY_300 and '__unicode__', 'unicode()'),
)))


if __name__ == '__main__':
    help(see)



#!/usr/bin/env python
# coding: utf-8
from __future__ import unicode_literals
"""
Unit tests that require Unicode string literals in Python 2.

"""
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import see


class TestSeeUnicode(unittest.TestCase):

    def test_char_width(self):
        # Arrange
        char_ascii = 'a'   # narrow char
        char_accent = 'á'  # narrow char
        char_combo = 'q̇'   # narrow char + zero-width combining char
        char_cjk = '猫'    # wide character

        # Act
        len_ascii = len(char_ascii)
        len_accent = len(char_accent)
        len_combo = len(char_combo)
        len_cjk = len(char_cjk)
        width_ascii = see.display_len(char_ascii)
        width_accent = see.display_len(char_accent)
        width_combo = see.display_len(char_combo)
        width_cjk = see.display_len(char_cjk)

        # Assert
        self.assertEqual(len_ascii, 1)
        self.assertEqual(len_accent, 1)
        self.assertEqual(len_combo, 2)
        self.assertEqual(len_cjk, 1)
        self.assertEqual(width_ascii, 1)
        self.assertEqual(width_accent, 1)
        self.assertEqual(width_combo, 1)
        self.assertEqual(width_cjk, 2)

    def test_display_len(self):
        # Arrange
        attr_ascii = '.hello_world()'
        attr_cyrillic = '.hello_мир()'
        attr_cjk = '.hello_世界()'
        attr_combo = '.hello_q̇()'
        diff_cjk = 2 if see.PY_300 else 0
        diff_combo = -1 if see.PY_300 else 0

        # Act
        width_ascii = see.display_len(attr_ascii)
        width_cyrillic = see.display_len(attr_cyrillic)
        width_cjk = see.display_len(attr_cjk)
        width_combo = see.display_len(attr_combo)
        justify_ascii = len(see.justify_token(attr_ascii, 20))
        justify_cyrillic = len(see.justify_token(attr_cyrillic, 20))
        justify_cjk = len(see.justify_token(attr_cjk, 20))
        justify_combo = len(see.justify_token(attr_combo, 20))

        # Assert
        self.assertEqual(width_ascii, 14)
        self.assertEqual(width_cyrillic, 12)
        self.assertEqual(width_cjk, 13)
        self.assertEqual(width_combo, 10)
        self.assertEqual(justify_cyrillic, justify_ascii)
        self.assertEqual(justify_cjk, justify_ascii - diff_cjk)
        self.assertEqual(justify_combo, justify_ascii - diff_combo)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import sys
from setuptools import setup

tests_require = [
    'mock>=2.0.0' if sys.version_info < (3, 3) else None,
    'unittest2>=1.1.0' if sys.version_info < (2, 7) else None,
]

setup(name='see',
      version='1.3.2',
      description='dir for humans',
      author='Liam Cooke',
      author_email='see@araile.com',
      license='BSD License',
      url='http://araile.github.io/see/',
      py_modules=['see'],
      install_requires=[],
      test_suite='tests',
      tests_require=[lib for lib in tests_require if lib],
      zip_safe=True,
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: Freely Distributable',
          'License :: OSI Approved :: BSD License',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
          'Topic :: Software Development',
      ])
